
# Cyto2SnpMatrix
# -----------------------------------------------------------------------------
# 24/12/2013

#Cyto2SnpMatrix(cychp.files="APT", annotation.file=paste0(cyhd.analysisPath(), "/", cyhd.annotation()$csv), cychp.attime=10, verbose=TRUE, output.type="snpmatrix", output.name="TEA2", sa=TRUE)

Cyto2SnpMatrix <- function(cychp.files, annotation.file, cychp.attime = 4,
    output.type = "snpmatrix", output.name = NA, sa = FALSE, verbose = FALSE) {

    # SETUP
    OptionsAffy.Init("Cyto2SnpMatrix", verbose)
    OptionsAffy.Cores(cychp.attime)
    # /SETUP

    # SYSTEM
    CheckSystem(bits=FALSE)
    # /SYSTEM

    # FUN CHECKS
    if( cychp.files == "" ) {
        cychp.files = "./";
    }
    output.type <- value(output.type, c("snpmatrix", "plink"))
    if (is.na(output.type)) {
        stop("Argument 'output.type' must be 'snpmatrix' or 'plink'.")
    }
    if (output.type == "plink" & is.na(output.name)) {
        stop("If 'output.type' is 'plink', 'output.name' must be filled.")
    }
    files.complete <- list.files(cychp.files, pattern=".cyhd.cychp.txt", 
        full.names=TRUE)
    if (length(files.complete) == 0) {
        stop("No 'cychp' files in folder '", cychp.files, "'.")
    }
    # /FUN CHECKS

    output.temp <- RandomName(prefix="tmp.cyto.snpmatrix.")

    # ARGS
    m <- paste0( "\n\n",
        " - Input:        ", cychp.files, "\n",
        " - Annotation:   ", annotation.file, "\n",
        " - Temp path:    ", output.temp, "\n",
        " - cychp x time: ", cychp.attime, "\n",
        " - Output.t:     ", output.type, "\n",
        " - Output.f:     ", output.name, "\n");
    ScreenAffy(m)
    # /ARGS

    # CLEAN
    unlink(output.name, recursive=TRUE, force=TRUE)
    unlink(output.temp, recursive=TRUE, force=TRUE)
    dir.create(output.temp, showWarnings=FALSE)
    # /CLEAN

    # CALLING
    ScreenAffy("Calling individuals.")
    calls.ind <<- plapply(X=files.complete, FUN=CytoProcess.SnpMatrix,
        output.temp = output.temp)
    
    OptionsAffy.CheckErrors()
    # /CALLING

    # ANNOTATION
    ScreenAffy("Loading annotation file.")
    annot <- tryCatch({
            read.csv(annotation.file, row.names=1, comment.char="#", 
                header=TRUE)
        }, warnning=function(w) {
            stop("Invalid file provided in \"annotation.file\".\n",
            w, "\n")
        }, error=function(e) { 
            stop("Invalid file provided in \"annotation.file\".\n",
            e, "\n")
        })
    # /ANNOTATION

    # FILTERING CALLS, GET COMMON
    ScreenAffy("Filtering calls.")
    common <- tryCatch({
            Reduce(intersect, 
                Map(function(x) { x$names }, calls.ind))
        }, warnning=function(w) {
            stop(w, "\n", "class(calls.ind): ", class(calls.ind), "\n",
                "class(calls.ind[1]): ", class(calls.ind[1]))
        }, error=function(e) { 
            stop(e, "\n", "class(calls.ind): ", class(calls.ind), "\n",
                "class(calls.ind[1]): ", class(calls.ind[1]))
        })
    calls <- Map(function(x){ x$genotype[common] }, calls.ind)
    # /FILTERING CALLS, GET COMMON

    # FLATTING CALLS TO SINGLE OBJECT
    ScreenAffy("Flatting data.")
    calls.ind <- data.frame(calls[[1]])
    if (length(calls) > 1) {
        for(ii in 2:length(calls)) {
            calls.ind <- cbind(calls.ind, calls[[ii]])
        }
    }
    rm("calls")
    # FLATTING CALLS TO SINGLE OBJECT


    if (output.type == "snpmatrix") {
        ScreenAffy(
            "Processing individuals to generate a 'SnpMatrix Container'."
        )

        # CREATING MAP
        map <- data.frame(chromosome=annot$Chromosome,
            snp.name=annot$dbSNP.RS.ID, 
            cM=rep(NA, length(annot$dbSNP.RS.ID)),
            position=as.numeric(annot$Physical.Position),
            allele.1=annot$Allele.A,
            allele.2=annot$Allele.B )
        rownames(map) <- rownames(annot)
        rm("annot")
        # /CREATING MAP

        # NAMING CALLS
        colnames(calls.ind) <- gsub(".cyhd.cychp.txt", "", 
            basename(files.complete))
        rownames(calls.ind) <- common
        rm("common")
        # /NAMING CALLS

        # COERCION
        ScreenAffy("Coercing to a SnpMatrix.")
        calls.sm <- new("SnpMatrix", t(as(calls.ind, "matrix")))
        rm("calls.ind")
        # /COERCION

        ScreenAffy("Creating container.")
        object <- list()
        object$map <- map
        object$genotype <- calls.sm
        object$map <- object$map[colnames(object$genotype), ]
        object$map <- object$map[order(object$map$chromosome, object$map$position), ]

    } else {
        ScreenAffy(
            "Processing individuals to generate a 'TPED' file."
        )

        strand <- as.character(annot$Strand)
        allele.A <- as.character(annot$Allele.A)
        allele.B <- as.character(annot$Allele.B)

        ScreenAffy("Getting A and B alleles.")
        allComplA <- c("A", "C", "G", "T")[ 
            match(allele.A, c( "T", "G", "C", "A"))
        ]
        forwA <- ifelse(strand=="+", allele.A, allComplA)

        allComplB <- c("A", "C", "G", "T")[ 
            match( allele.B, c("T", "G", "C", "A")) 
        ]
        forwB <- ifelse(strand == "+", allele.B, allComplB)
        rm("allele.A", "allele.B", "strand")

        ScreenAffy("Creating CALLS dataset." )
        alleleA <- alleleB <- matrix("", nrow=nrow(calls.ind), 
            ncol=ncol(calls.ind), dimnames=dimnames(calls))
        for( ii in 1:ncol(calls.ind)) {
            alleleA[ , ii] <- ifelse(as.numeric(calls.ind[ , ii]) < 3,
                forwA, forwB)
            alleleB[ , ii] <- ifelse(as.numeric(calls.ind[ , ii]) < 2, 
                forwA, forwB)
        }

        ScreenAffy("Coercing TPED to matrix.")
        p1 <- seq(1, ncol(calls.ind)*2, 2)
        p2 <- seq(2, ncol(calls.ind)*2, 2)

        tped <- matrix("", ncol=ncol(calls.ind) * 2,
            nrow=nrow(calls.ind))
        
        for(ii in 1:ncol(calls.ind)) {
            tped[ , p1[ii]] <- alleleA[ , ii]
            tped[ , p2[ii]] <- alleleB[ , ii]
        }

        rownames(tped) <- common
        rm("common", "calls.ind", "alleleA", "alleleB", "forwA", "forwB")

        ScreenAffy("Selecting SNP data from annotation file.")
        annot <- annot[rownames(tped), ]
        tped.an <- data.frame(annot$Chromosome, annot$dbSNP.RS.ID, 
            as.numeric(annot$ChrX.pseudo.autosomal.region.1), 
            as.numeric(annot$Physical.Position)
        )

        tped <- cbind(tped.an, tped)
        ScreenAffy("Writing TPED file to disk '", output.name, ".tped'.")
        write.table(tped, file=paste0(output.name, ".tped"), quote=FALSE, 
            row.names=FALSE, col.names=FALSE)

        rm("tped")
        object <- NA
    }

    unlink(output.temp, recursive=TRUE, force=TRUE)
    return(object)
}
