
# visualization
#   -> population
#       -> single SNP for all (calling)
#   -> individual
#       -> all SNP (calling)
#       -> strength vs. contrast

Cyto2QCView <- function(path, visualization, individual = NA, verbose = FALSE) {
    visualization <- value(visualization, c("snp", "int", "sc"))
    if (is.na(visualization)) {
        stop("Wrong type of 'visualization'. Available are: 'snp', 'calling' and 'sc'.")
    }

    res <- list()
    res$location <- path
    res$type <- visualization
    res$individual <- individual
    res$verbose <- verbose

    class(res) <- "Cyto2QCView"

    return(res)
}

plotIndIntensities <- function(gnds, individual, verbose) {
    # Individual Intensities
    plot1 <- ggplot(data = gnds, aes(x=log2(ASignal), y=log2(BSignal), color=Call)) + geom_point(shape = 20, size=1)
    plot1 <- plot1 + scale_color_manual(name = "GType", labels = c("AA", "AB", "BB", "NC"), values = c("#008000", "#FF6347", "#87CEEB", "black"))
    plot1 <- plot1 + ggtitle(paste0("ASignal vs. BSignal for ", individual))
    #plot1 <- plot1 + expand_limits(x = 0, y = 0)
    return(plot1)
}

plotIndStrength <- function(gnds, individual, verbose) {
    # Individual Strength vs. Contrast
    plot2 <- ggplot(data = gnds, aes(x=Contrast, y=SignalStrength, color=Call)) + geom_point(shape = 20, size=1)
    plot2 <- plot2 + scale_color_manual(name = "GType", labels = c("AA", "AB", "BB", "NC"), values = c("#008000", "#FF6347", "#87CEEB", "black"))
    plot2 <- plot2 + ggtitle(paste0("Strength vs. Contrast for ", individual))
    #plot2 <- plot2 + expand_limits(x = 0, y = 0)
    return(plot2)
}

plotSnpIntensities <- function(path, individual, verbose) {
    # Population
    gnds.files <- list.files(path, full.names=T, pattern=".genotype.f.txt")
    
    dta <- mclapply(gnds.files, function(x) {
        message( x )
        gnds <- fread(x, header = TRUE)
        as.data.frame(gnds[gnds[, ProbeSetName==individual],])
    })
    names(dta) <- list.files("cychp/", pattern=".genotype.f.txt")
    
    dta <- data.table(do.call("rbind", dta))

    plot3 <- ggplot(data = dta, aes(x=log2(ASignal), y=log2(BSignal), color=Call)) + geom_point(shape = 20, size=1)
    plot3 <- plot3 + scale_color_manual(name = "GType", labels = c("AA", "AB", "BB", "NC"), values = c("#008000", "#FF6347", "#87CEEB", "black"))
    plot3 <- plot3 + ggtitle(paste0("ASignal vs. BSignal for ", individual))
    #plot3 <- plot3 + expand_limits(x = 0, y = 0)
    return(plot3)
}


plot.Cyto2QCView <- function(x) {
    plot <- NA
    if (x$verbose) {
        cat("[plot] Type:", x$type, "\n")
    }

    output.path <- RandomName(prefix="tmp.cyto.plotQc.")
    dir.create(output.path, showWarnings=FALSE)

    if (x$type == "snp") {
        if (x$verbose) {
            cat("[plot] Location: ", x$location, "\n")
            cat("[plot] SNP: ", x$individual, "\n")
        }

        if (x$verbose) {
            cat("[plot] Parsing individuals.\n")
        }
        
        # PARSING ALL DATA-FILES
        files <- list.files(x$location, full.name=T, pattern=".cyhd.cychp.txt")
        
        
        parse <- function(x) {
            pypa <- system.file("exec/parser.py", package="affy2sv")
            pypa <- paste0("python ", pypa, " -i ", f,
                " -o ", output.path)
            if (x$verbose) {
                cat(pypa, "\n")
            }
            system(pypa)
        }
        plapply(files, parse)
        # /

        plot <- plotSnpIntensities(output.path, x$individual, x$verbose)
    }
    if (x$type == "int") {
        gnds.path <- paste0(x$location, .Platform$file.sep, x$individual, collapse='', sep='')

        if (x$verbose) {
            cat("[plot] Parsing file '", gnds.path, "' to '", output.path, "'\n")
        }

        # PARSING SINGLE FILE
        pypa <- system.file("exec/parser.py", package="affy2sv")
        pypa <- paste0("python ", pypa, " -i ", gnds.path,
                 " -o ", output.path)
        system(pypa)
        # /

        f.f <- strsplit(x$individual, "\\.")[[1]]
        f.f <- paste0(paste0(f.f[-length(f.f)], collapse="", sep="."), "genotype.f.txt")
        f.f <- paste0(output.path, .Platform$file.sep, f.f, collapse="", sep="")

        if (x$verbose) {
            cat("[plot] Location: ", gnds.path, "\n")
            cat("[plot] Individual: ", x$individual, "\n")
            cat("[plot] Genotype: ", f.f, "\n")
        }
        plot <- plotIndIntensities(gnds, x$individual, x$verbose)
    }
    if (x$type == "sc") {
        gnds.path <- paste0(x$location, .Platform$file.sep, x$individual, collapse='', sep='')

        if (x$verbose) {
            cat("[plot] Parsing file '", gnds.path, "' to '", output.path, "'\n")
        }

        # PARSING SINGLE FILE
        pypa <- system.file("exec/parser.py", package="affy2sv")
        pypa <- paste0("python ", pypa, " -i ", gnds.path,
                 " -o ", output.path)
        system(pypa)
        # /

        f.f <- strsplit(x$individual, "\\.")[[1]]
        f.f <- paste0(paste0(f.f[-length(f.f)], collapse="", sep="."), "genotype.f.txt")
        f.f <- paste0(output.path, .Platform$file.sep, f.f, collapse="", sep="")

        if (x$verbose) {
            cat("[plot] Location: ", gnds.path, "\n")
            cat("[plot] Individual: ", x$individual, "\n")
            cat("[plot] Genotype: ", f.f, "\n")
        }
        plot <- plotIndStrength(gnds, x$individual, x$verbose)
    }

    if (x$verbose) {
        cat("[plot] Cleaning.\n")
    }
    unlink(output.path, recursive=TRUE, force=TRUE)

    return(plot)
}


print.Cyto2QCView <- function(x) {
    cat("Object of class Cyto2QCView\n")
    cat("Visualization '", x$type, "'\n")

    if (x$type == "snp") {
        cat(" for all individuals in '", x$location, "'\n")
    }
    else {
        cat(" for all individual '", x$individual, "'\n")
    }
}




#   library(affy2sv); t <- Cyto2QCView("cychp.f", "snp", individual="S-4RGHR", verose=T);
#   library(affy2sv); t <- Cyto2QCView("cychp", "int", individual="CyHD_011612K_SS1038_401136LB.cyhd.cychp.txt", verbose=T);affy2sv.unload()



    
#     apds <- read.delim("CyHD_011612K_SS1036_400520FM.cyhd.cychp.apds.f.txt", header=TRUE)


#     files <- list.files(".", full.name=T, pattern=".cyhd.cychp.txt")
#     output.path <- "cychp/"
#     for( f in files ) {
#         pypa <- system.file("exec/parser.py", package="affy2sv")
#         pypa <- paste0("python ", pypa, " -i ", f,
#             " -o ", "cychp/")
#         print( pypa )
#         system(pypa)
#     }





#     individual <- "S-4RGHR"

#     gnds.files <- list.files("cychp/", full.names=T, pattern=".genotype.f.txt")
#     dta <- mclapply(gnds.files, function(x) {
#         message( x )
#         gnds <- fread(x, header = TRUE)
#         as.data.frame(gnds[gnds[, ProbeSetName==individual],])
#     })
#     names(dta) <- list.files("cychp/", pattern=".genotype.f.txt")

#     dta <- data.table(do.call("rbind", dta))

#     plot1 <- ggplot(data = dta, aes(x=log2(ASignal), y=log2(BSignal), color=Call)) + geom_point(shape = 20, size=1)
#     plot1 <- plot1 + scale_color_manual(name = "GType", labels = c("AA", "AB", "BB", "NC"), values = c("#90EE90", "#FF6347", "#87CEEB", "black"))
#     plot1 <- plot1 + ggtitle(paste0("ASignal vs. BSignal for ", individual))
#     return plot1
# }